@@ -6,7 +6,7 @@ pub enum Command {
     NextTrack,
     PreviousTrack,
     ResumePause,
-    PlayContext,
+    PlayRandom,
     Repeat,
     Shuffle,
     VolumeUp,
@@ -29,18 +29,16 @@ pub enum Command {
     SwitchDevice,
     SearchContext,
 
+    ShowActionsOnSelectedItem,
+    ShowActionsOnCurrentTrack,
+
     BrowseUserPlaylists,
     BrowseUserFollowedArtists,
     BrowseUserSavedAlbums,
 
-    BrowsePlayingTrackArtists,
-    BrowsePlayingTrackAlbum,
     BrowsePlayingContext,
-    BrowseSelectedTrackArtists,
-    BrowseSelectedTrackAlbum,
 
     SearchPage,
-
     PreviousPage,
 
     SortTrackByTitle,
@@ -51,13 +49,23 @@ pub enum Command {
     ReverseTrackOrder,
 }
 
+/// An action on a Spotify item (track,album,artist,playlist)
+#[derive(Copy, Clone, Debug)]
+pub enum Action {
+    AddTrackToPlaylist,
+    SaveToLibrary,
+    BrowseArtist,
+    BrowseAlbum,
+    BrowseRecommendations,
+}
+
 impl Command {
     pub fn desc(&self) -> &'static str {
         match self {
             Self::NextTrack => "next track",
             Self::PreviousTrack => "previous track",
             Self::ResumePause => "resume/pause based on the current playback",
-            Self::PlayContext => "play a random track in the current context",
+            Self::PlayRandom => "play a random track in the current context",
             Self::Repeat => "cycle the repeat mode",
             Self::Shuffle => "toggle the shuffle mode",
             Self::VolumeUp => "increase playback volume",
@@ -71,6 +79,8 @@ impl Command {
             }
             Self::ChooseSelected => "choose the selected item and act on it",
             Self::RefreshPlayback => "manually refresh the current playback",
+            Self::ShowActionsOnSelectedItem => "show actions on a selected item",
+            Self::ShowActionsOnCurrentTrack => "show actions on the currently playing track",
             Self::FocusNextWindow => "focus the next focusable window (if any)",
             Self::FocusPreviousWindow => "focus the previous focusable window (if any)",
             Self::SwitchTheme => "open a popup for switching theme",
@@ -79,15 +89,7 @@ impl Command {
             Self::BrowseUserPlaylists => "open a popup for browsing user's playlists",
             Self::BrowseUserFollowedArtists => "open a popup for browsing user's followed artists",
             Self::BrowseUserSavedAlbums => "open a popup for browsing user's saved albums",
-            Self::BrowsePlayingTrackArtists => {
-                "open a popup for browsing current playing track's artists"
-            }
-            Self::BrowsePlayingTrackAlbum => "browse the current playing track's album",
             Self::BrowsePlayingContext => "browse the current playing context",
-            Self::BrowseSelectedTrackArtists => {
-                "open a popup for browsing the selected track's artists"
-            }
-            Self::BrowseSelectedTrackAlbum => "browse to the selected track's album",
             Self::SearchPage => "go to the search page",
             Self::PreviousPage => "go to the previous page",
             Self::SortTrackByTitle => "sort the track table (if any) by track's title",
