@@ -1,3 +1,5 @@
+use event::ClientRequest;
+
 mod auth;
 mod client;
 mod command;
@@ -18,7 +20,7 @@ async fn main() -> anyhow::Result<()> {
 
     // parse command line arguments
     let matches = clap::App::new("spotify-player")
-        .version("0.3.0")
+        .version("0.4.0")
         .about("A command driven spotify player")
         .author("Thang Pham <phamducthang1234@gmail>")
         .arg(
@@ -62,18 +64,32 @@ async fn main() -> anyhow::Result<()> {
         std::fs::create_dir_all(&cache_audio_folder)?;
     }
 
-    // init the shared application state
+    // initialize the application state
     let mut state = state::State::default();
+    // parse config options from the config files into application's state
     state.parse_config_files(&config_folder, matches.value_of("theme"))?;
     let state = std::sync::Arc::new(state);
 
+    // initialize a librespot session
+    let session = auth::new_session(&cache_folder, state.app_config.device.audio_cache).await?;
+
     // start application's threads
     let (send, recv) = std::sync::mpsc::channel::<event::ClientRequest>();
 
-    let session = auth::new_session(&cache_folder, state.app_config.device.audio_cache).await?;
-    // get the auth token and store it inside the player state
-    let token = token::get_token(&session, &state.app_config.client_id).await?;
-    state.player.write().unwrap().token = token;
+    // init thread to get necessary data to render the player on startup
+    std::thread::spawn({
+        let send = send.clone();
+        move || {
+            send.send(ClientRequest::GetCurrentUser).unwrap();
+            send.send(ClientRequest::GetCurrentPlayback).unwrap();
+            send.send(ClientRequest::GetUserPlaylists).unwrap();
+
+            // wait for 1 second before getting the device list
+            // because it may take time to intialize the integrated Spotify client
+            std::thread::sleep(std::time::Duration::from_millis(1000));
+            send.send(ClientRequest::GetDevices).unwrap();
+        }
+    });
 
     // connection thread (used to initialize the integrated Spotify client using librespot)
     #[cfg(feature = "streaming")]
@@ -88,7 +104,8 @@ async fn main() -> anyhow::Result<()> {
     // client event handler thread
     std::thread::spawn({
         let state = state.clone();
-        let client = client::Client::new();
+        let client = client::Client::new(session, state.app_config.client_id.clone());
+        client.init_token().await?;
         move || {
             client::start_client_handler(state, client, recv);
         }
@@ -108,10 +125,10 @@ async fn main() -> anyhow::Result<()> {
         let send = send.clone();
         let state = state.clone();
         move || {
-            client::start_player_event_watchers(state, send, session);
+            client::start_player_event_watchers(state, send);
         }
     });
 
     // application's UI as the main thread
-    ui::start_ui(state)
+    ui::start_ui(state, send)
 }
