@@ -2,6 +2,7 @@ use super::player::*;
 use crate::{config, key};
 
 use tui::widgets::{ListState, TableState};
+
 pub type UIStateGuard<'a> = std::sync::MutexGuard<'a, UIState>;
 
 // TODO: improve the documentation for UI states' struct
@@ -24,8 +25,9 @@ pub struct UIState {
 #[derive(Clone, Debug)]
 pub enum PageState {
     CurrentPlaying,
-    Browsing(String),
+    Browsing(ContextId),
     Searching(String, Box<SearchResults>),
+    Recommendations(SeedItem, Option<Vec<Track>>),
 }
 
 /// Window state
@@ -40,6 +42,8 @@ pub enum WindowState {
     Artist(TableState, ListState, ListState, ArtistFocusState),
     /// tracks, albums, artists, playlists
     Search(ListState, ListState, ListState, ListState, SearchFocusState),
+    /// tracks
+    Recommendations(TableState),
 }
 
 /// Popup state
@@ -47,12 +51,20 @@ pub enum WindowState {
 pub enum PopupState {
     CommandHelp(usize),
     ContextSearch(String),
-    UserPlaylistList(ListState),
+    UserPlaylistList(PlaylistPopupAction, Vec<Playlist>, ListState),
     UserFollowedArtistList(ListState),
     UserSavedAlbumList(ListState),
     DeviceList(ListState),
     ArtistList(Vec<Artist>, ListState),
     ThemeList(Vec<config::Theme>, ListState),
+    ActionList(Item, ListState),
+}
+
+/// An action on a playlist popup list
+#[derive(Debug)]
+pub enum PlaylistPopupAction {
+    Browse,
+    AddTrack(TrackId),
 }
 
 /// A trait representing a focusable state
@@ -95,10 +107,7 @@ impl UIState {
     }
 
     /// gets a list of items possibly filtered by a search query if currently inside a search state
-    pub fn get_search_filtered_items<'a, T: std::fmt::Display>(
-        &self,
-        items: &'a [T],
-    ) -> Vec<&'a T> {
+    pub fn filtered_items_by_search<'a, T: std::fmt::Display>(&self, items: &'a [T]) -> Vec<&'a T> {
         match self.popup {
             Some(PopupState::ContextSearch(ref query)) => items
                 .iter()
@@ -107,6 +116,12 @@ impl UIState {
             _ => items.iter().collect::<Vec<_>>(),
         }
     }
+
+    /// creates to a new page
+    pub fn new_page(&mut self, page: PageState) {
+        self.history.push(page);
+        self.popup = None;
+    }
 }
 
 impl Default for UIState {
@@ -127,34 +142,36 @@ impl Default for UIState {
 
 impl PopupState {
     /// gets the state of the current list popup
-    pub fn get_list_state(&self) -> Option<&ListState> {
+    pub fn list_state(&self) -> Option<&ListState> {
         match self {
             Self::DeviceList(ref state) => Some(state),
-            Self::UserPlaylistList(ref state) => Some(state),
+            Self::UserPlaylistList(_, _, ref state) => Some(state),
             Self::UserFollowedArtistList(ref state) => Some(state),
             Self::UserSavedAlbumList(ref state) => Some(state),
             Self::ArtistList(_, ref state) => Some(state),
             Self::ThemeList(_, ref state) => Some(state),
+            Self::ActionList(_, ref state) => Some(state),
             Self::CommandHelp(_) | Self::ContextSearch(_) => None,
         }
     }
 
     /// gets the (mutable) state of the current list popup
-    pub fn get_list_state_mut(&mut self) -> Option<&mut ListState> {
+    pub fn list_state_mut(&mut self) -> Option<&mut ListState> {
         match self {
             Self::DeviceList(ref mut state) => Some(state),
-            Self::UserPlaylistList(ref mut state) => Some(state),
+            Self::UserPlaylistList(_, _, ref mut state) => Some(state),
             Self::UserFollowedArtistList(ref mut state) => Some(state),
             Self::UserSavedAlbumList(ref mut state) => Some(state),
             Self::ArtistList(_, ref mut state) => Some(state),
             Self::ThemeList(_, ref mut state) => Some(state),
+            Self::ActionList(_, ref mut state) => Some(state),
             Self::CommandHelp(_) | Self::ContextSearch(_) => None,
         }
     }
 
     /// returns the selected position in the current list popup
     pub fn list_selected(&self) -> Option<usize> {
-        match self.get_list_state() {
+        match self.list_state() {
             None => None,
             Some(state) => state.selected(),
         }
@@ -162,7 +179,7 @@ impl PopupState {
 
     /// selects a position in the current list popup
     pub fn list_select(&mut self, id: Option<usize>) {
-        match self.get_list_state_mut() {
+        match self.list_state_mut() {
             None => {}
             Some(state) => state.select(id),
         }
@@ -171,11 +188,12 @@ impl PopupState {
 
 impl WindowState {
     /// gets the state of the context track table
-    pub fn get_track_table_state(&mut self) -> Option<&mut TableState> {
+    pub fn track_table_state(&mut self) -> Option<&mut TableState> {
         match self {
             Self::Playlist(ref mut state) => Some(state),
             Self::Album(ref mut state) => Some(state),
-            Self::Artist(ref mut top_tracks, _, _, _) => Some(top_tracks),
+            Self::Artist(ref mut state, _, _, _) => Some(state),
+            Self::Recommendations(ref mut state) => Some(state),
             _ => None,
         }
     }
@@ -209,6 +227,7 @@ impl WindowState {
                 ArtistFocusState::Albums => albums.select(id),
                 ArtistFocusState::RelatedArtists => related_artists.select(id),
             },
+            Self::Recommendations(ref mut state) => state.select(id),
         }
     }
 
@@ -233,6 +252,7 @@ impl WindowState {
                 ArtistFocusState::Albums => albums.selected(),
                 ArtistFocusState::RelatedArtists => related_artists.selected(),
             },
+            Self::Recommendations(ref state) => state.selected(),
         }
     }
 }
